/**************************************************************************
* obtype.c:	Main program for ObType.
*		Describe the contents of an Oberheim patch file.
*		A part of OberSuite for the Commodore Amiga.
*
* Author:	Daniel Barrett, barrett@cs.umass.edu.
* Version:	1.0.
* Copyright:	None!  This program is in the Public Domain.
*		Please share it with others.
***************************************************************************/

#include "decl.h"
#include "obtype.h"

char *version = "$VER: ObType "  VERSION " " VERDATE;

/***************************************************************************
* Main program.
***************************************************************************/

main(argc, argv)
int argc; char *argv[];
{
	int i;

	strcpy(programName, BaseName(argv[0]));	/* Global variable. */
	thePrintStyle = PRETTY;			/* Global variable. */

	if (argc == 1)
	{
		ShortUsageMsg();
		BegForUsage();
	}
	else if ((argc == 2) && (!strcmp(argv[1], "?")))
		DetailedUsage();
	else if (!HandleOptions(argc, argv))
		BegForUsage();
	else if (optind > argc-1)
		ErrorMsg(ERROR_NUMARGS);
	else
		for (i=optind; i<argc; i++)
			ObType(argv[i]);

	exit(0);	/* Make a real exit code! */
}


/*
 * Given the filename, examine its contents and print the information.
 */

void ObType(char *filename)
{
	PATCHINFO pi;

	InitPatchInfo(&pi);
	pi.source = PI_SOURCE_FILE;

	if (!LookAtFileSize(&pi, filename))
		;
	else if (!AllocPatchInfo(&pi))
		ErrorMsg(ERROR_MALLOC);
	else if (!GetPatchFromFile(&pi, filename))
		ErrorMsg(ERROR_GETFAILED);
	else if (!FigureThingsOutFromHeader(&pi))
		;
	else
		PrintStuff(&pi, filename);;

	FreePatchInfo(&pi);
}


/*
 * Depending on the print style chosen by the user, call the appropriate
 * printing function "fcn".
 */

void PrintStuff(PATCHINFO *pi, char *filename)
{
	void (*fcn)();

	switch (thePrintStyle)
	{
		case SILENT:
			fcn = VerifyStuffOnly;
			break;
		case SHORT:
			fcn = PrintStuffBriefly;
			break;
		case PRETTY:
			if (pi->numPatches <= 1)
				fcn = PrintStuffLong;
			else
			{
				PrintHeader(pi, filename);
				fcn = PrintStuffNicely;
			}
			break;
		case VERBOSE:
			PrintHeader(pi, filename);
			fcn = PrintStuffLong;
			break;
		default:
			fcn = PrintStuffNicely;
			break;
	}

	PrintNames(fcn, pi);
}


/*
 * Given a printing function fcn(), print information about all patches
 * found in the given PATCHINFO structure.
 * Quit if an error is found in the patch data (by VerifyPatch()).
 */

void PrintNames(void (*fcn)(), PATCHINFO *pi)
{
	int counter = 0;
	long offset = 0L;
	BOOL success = TRUE;

	while ((counter < pi->numPatches) && success)
	{
		if (success = VerifyPatch(pi, offset))
			fcn(pi, offset, counter);
		offset += (long)pi->rightSize;
		counter++;
	}
}


/***************************************************************************
* Printing functions of various styles.
***************************************************************************/

/* Do nothing.  No output required. */

void VerifyStuffOnly(PATCHINFO *pi, long offset, int counter)
{
	return;
}

	
void PrintStuffNicely(PATCHINFO *pi, long offset, int counter)
{
	UBYTE num;

	num = pi->data[offset + BYTE_PATCHNUMBER];
	printf("%02d: ", num);

	PrintPatchName(pi->data + offset, pi->nameOffset);
    	counter++;

	if ((counter % NAMES_PER_ROW == 0) || (counter == pi->numPatches))
		putchar('\n');
	else
		printf("%s", SPACE_BETWEEN_NAMES);
}


void PrintStuffBriefly(PATCHINFO *pi, long offset, int counter)
{
	PrintPatchName(pi->data + offset, pi->nameOffset);
	putchar('\n');
}


void PrintStuffLong(PATCHINFO *pi, long offset, int counter)
{
	PrintPatchInfo(pi, offset);
}


void PrintHeader(PATCHINFO *pi, char *filename)
{
	if (pi->numPatches > 1)
    		printf("%sFile %s contains %d %s patches:%s\n", 
			HILITE_ON, filename, pi->numPatches, pi->modeName,
			HILITE_OFF);
	else
		printf("%sFile %s:%s\n", HILITE_ON, filename, HILITE_OFF);
}

	
BOOL HandleOptions(int argc, char *argv[])
{
	int c;
	short printed = 0;

	while ((c = getopt(argc, argv, options)) != EOF)
	{
		switch (c)
		{
			case OPT_SILENT:
				thePrintStyle = SILENT;
				printed++;
				break;
			case OPT_SHORT:
				thePrintStyle = SHORT;
				printed++;
				break;
			case OPT_PRETTY:
				thePrintStyle = PRETTY;
				printed++;
				break;
			case OPT_VERBOSE:
				thePrintStyle = VERBOSE;
				printed++;
				break;
			default:
				return(FALSE);
		}
	}

	if (printed > 1)
		ErrorMsg(ERROR_TWOPRINTS);

	return(printed <= 1);
}

	
/*****************************************************************************/


void ShortUsageMsg(void)
{
	if (!ERR_OUTPUT_ALLOWED)
		return;

	fprintf(stderr, "Usage: %s [output_option] file1 file2 ...\n",
		programName);
}


void UsageMsg(void)
{
	if (!ERR_OUTPUT_ALLOWED)
		return;

	fprintf(stderr,
	    "%s identifies the Oberheim patch data in specified files.\n",
	    programName);
	ShortUsageMsg();

	fprintf(stderr, "\nLegal options are:\n");
	fprintf(stderr, "\t-%c:  Short output format\n", OPT_SHORT);
	fprintf(stderr, "\t-%c:  Medium output format %s\n", 
		OPT_PRETTY, "(DEFAULT, if no option is given.)");
	fprintf(stderr, "\t-%c:  Long output format\n", OPT_VERBOSE);
	fprintf(stderr, "\t-%c:  Quiet (no output if data is valid).\n",
			OPT_SILENT);

	fprintf(stderr,
		"Note:  The program behavior changes if there is more than"
		" 1 patch in a file.\n");

	fprintf(stderr, "\nExamples:\n");
	fprintf(stderr, "\t%s myPatchFile\n", programName);
	fprintf(stderr, "\t%s -%c myPatchFile\n", programName, OPT_SHORT);
	fprintf(stderr, "\t%s -%c oneFile anotherFile thirdFile\n",
			programName, OPT_VERBOSE);
}


char *Version(void)
{
	static char v[] = VERSION;
	return(v);
}
